:
# NAME:
#	newlog - rotate log files
#
# SYNOPSIS:
#	newlog.sh [options] "log"[:"num"] ...
#
# DESCRIPTION:
#	This script saves multiple generations of each "log".
#	The "logs" are kept compressed except for the current one and
#	the previous day.
#
#	Options:
#
#	-C "compress"
#		Compact old logs (other than .0) with "compress"
#		(default is 'gzip' or 'compress' if no 'gzip').
#
#	-E "ext"
#		If "compress" produces a file extention other than
#		'.Z' or '.gz' we need to know.
#
#	-G "gens"
#		"gens" is a comma separated list of "log":"num" pairs
#		that allows certain logs to handled differently.
#
#	-N	Don't actually do anything, just show us.
#
#	-e	Normally logs are only cycled if non-empty, this
#		option forces empty logs to be cycled as well.
#
#	-g "group"
#		Set the group of "log" to "group".
#
#	-m "mode"
#		Set the mode of "log" (default 664).
#
#	-M "mode"
#		Set the mode of old logs (default 444).
#
#	-n "num"
#		Keep "num" generations of "log".
#
#	-o "owner"
#		Set the owner of "log".
#
# BUGS:
#	'Newlog.sh' tries to avoid being fooled by symbolic links, but
#	multiply indirect symlinks are only handled on machines where
#	test(1) supports a check for symlinks. 
#
# AUTHOR:
#	Simon J. Gerraty <sjg@crufty.net>
#

# RCSid:
#	$Id: newlog.sh,v 1.12 2007/07/21 00:35:36 sjg Exp $
#
#	@(#) Copyright (c) 1993-2002 Simon J. Gerraty
#
#	This file is provided in the hope that it will
#	be of use.  There is absolutely NO WARRANTY.
#	Permission to copy, redistribute or otherwise
#	use this file is hereby granted provided that 
#	the above copyright notice and this notice are
#	left intact. 
#      
#	Please send copies of changes and bug-fixes to:
#	sjg@crufty.net
#

Mydir=`dirname $0`
case $Mydir in
/*) ;;
*) Mydir=`cd $Mydir; pwd`;;
esac

# places to find chown (and setopts.sh)
PATH=$PATH:/usr/etc:/sbin:/usr/sbin:/usr/local/share/bin:/share/bin:$Mydir

# linux doesn't necessarily have compress,
# and gzip appears in various locations...
Which() {
	case "$1" in
	-*) t=$1; shift;;
	*) t=-x;;
	esac
	case "$1" in
	/*)	test $t $1 && echo $1;;
	*)
		for d in `IFS=:; echo ${2:-$PATH}`
		do
			test $t $d/$1 && { echo $d/$1; break; }
		done
		;;
	esac
}

opt_n=7
opt_m=664
opt_M=444
opt_str=Nn:o:g:G:C:M:m:eE:

. setopts.sh

for x in $opt_C gzip compress
do
    opt_C=`Which $x "/bin:/usr/bin:$PATH"`
    test -x $opt_C && break
done

# sorry, setops semantics for booleans changed.
case "$opt_N" in
""|0)	ECHO=;;
*)	ECHO=echo;;
esac
case "$opt_e" in
""|0)	force=;;
*)	force=yes;;
esac

rotatelog()
{
	log=$1
	n=${2:-$opt_n}

	# make sure excess generations are trimmed
	$ECHO rm -f `echo $log.$n | sed 's/\([0-9]\)$/[\1-9]*/'`

	while test $n -gt 0 
	do
		p=`expr $n - 1`
		if test -s $log.$p; then
                        $ECHO rm -f $log.$p.*
                        $ECHO $opt_C $log.$p
                        if test "x$opt_M" != x; then
                                $ECHO chmod $opt_M $log.$p.* 2> /dev/null
			fi
		fi
		for ext in $opt_E .gz .Z ""
		do
			test -f $log.$p$ext || continue
			$ECHO mv $log.$p$ext $log.$n$ext
		done
		n=$p
	done
	# leave $log.0 uncompressed incase some one still has it open.
	$ECHO mv $log $log.0
	if test "x$opt_M" != x; then
	        $ECHO chmod $opt_M $log.0 2> /dev/null
	fi
	# someone may have managed to write to it already
	# so don't truncate it.
	$ECHO touch $log
	# the order here matters
	test "x$opt_o" = x || $ECHO chown $opt_o $log
	test "x$opt_g" = x || $ECHO chgrp $opt_g $log
	test "x$opt_m" = x || $ECHO chmod $opt_m $log
}

test $# -gt 0 || exit 0	# nothing to do.

# see whether test handles -h
case `test -h /dev/null 2>&1` in
"")	# it does
	test_h=yes
	logs="$*"
	;;
*)	# No, so this is about all we can do...
	logs=`/bin/ls -ld $* | awk '{ print $NF }'`
	test_h=
	;;
esac

for f in $logs
do
	case "$test_h" in
	yes)
		while test -h $f	# it is [still] a symlink
		do
			f=`/bin/ls -ld $f | awk '{ print $NF }'`
		done
		;;
	esac
	n=$opt_n
	case "$f" in
	*:[1-9]*)
		set -- `IFS=:; echo $f`; f=$1; n=$2;;
	esac
	case ",${opt_G}:" in
	*,${f}:*)
		# opt_G is a , separated list of log:n pairs
		n=`echo $opt_G | sed -e "s,.*${f}:\([0-9][0-9]*\).*,\1,"`
		n=${n:-$opt_n}
		;;
	esac
		
	test "$force" -o -s $f && rotatelog $f $n
done
